#ifndef MOUSEAPI_H
#define MOUSEAPI_H


#include "OrxonoxPrereqs.h"
#include "util/OgreForwardRefs.h"
#include "graphics/Camera.h"
#include <util/Math.h>
#include <list>
#include <core/input/InputHandler.h>
#include <graphics/Camera.h>
#include <core/GraphicsManager.h>
#include <core/input/InputState.h>
#include <OgreCamera.h>
#include <OgreViewport.h>
#include "CameraManager.h"
#include <functional>

namespace orxonox
{

typedef uint ClickableObjectID;
typedef uint ScrollableElementID;

class MouseAPI : public InputHandler, public Singleton<MouseAPI>
{
friend class Singleton<MouseAPI>;
private:

    struct clickableElement
    {
        //static ClickableObjectID lastClID;
        ClickableObjectID id;
        Vector3 position;
        float radius;
        std::list<MouseButtonCode::ByEnum> buttons;
        std::function<void(MouseButtonCode::ByEnum button)> onClickedFunction;
        clickableElement(ClickableObjectID id,const Vector3& position,float radius,const std::list<MouseButtonCode::ByEnum>& buttons,std::function<void(MouseButtonCode::ByEnum button)> onClickedFunction):position(position),
            radius(radius), buttons(buttons), onClickedFunction(onClickedFunction),id(id){}
    };

    struct scrollElement
    {
        static ScrollableElementID lastScID;
        ScrollableElementID id;
        bool considerPosition;
        Vector3 position;
        float radius;
        std::function<void(int abs,int rel,const IntVector2& mousePos)> onScrolledFunction;
        scrollElement(ScrollableElementID id,std::function<void(int abs,int rel,const IntVector2& mousePos)> onScrolledFunction):considerPosition(false),
            onScrolledFunction(onScrolledFunction),id(id){}
        scrollElement(ScrollableElementID id,const Vector3& position, float radius, std::function<void(int abs,int rel,const IntVector2& mousePos)> onScrolledFunction):considerPosition(true),
            position(position), radius(radius), onScrolledFunction(onScrolledFunction),id(id){}
    };

    static MouseAPI* singletonPtr_s;
    std::list<clickableElement> clickEvents;
    std::list<scrollElement> scrollEvents;
    Ogre::Camera *cam ;
    IntVector2 mousePos;
    InputState* state;
    bool active = false;



public:

    MouseAPI();
    ~MouseAPI();
    virtual void buttonPressed (MouseButtonCode::ByEnum button) override;
    virtual void buttonReleased(MouseButtonCode::ByEnum button)  override{}
    virtual void buttonHeld    (MouseButtonCode::ByEnum button) override{}
    virtual void mouseMoved    (IntVector2 abs, IntVector2 rel, IntVector2 clippingSize) override;
    virtual void mouseScrolled (int abs, int rel) override;

    ClickableObjectID addClickableObject(const Vector3& position,float radius,const std::list<MouseButtonCode::ByEnum>& buttons,std::function<void(MouseButtonCode::ByEnum button)>  onClickedFunction);
    ScrollableElementID addScrollElement(const Vector3& position,float radius,std::function<void(int abs,int rel,const IntVector2& mousePos)> onScrolledFunction);
    ScrollableElementID addScrollElement(std::function<void(int abs,int rel,const IntVector2& mousePos)> onScrolledFunction);

    void changePositionOfClickableObject(ClickableObjectID id,const Vector3& position);
    void changePositionOfScrollableElement(ScrollableElementID id,const Vector3& position);
    void changeRadiusOfClickableObject(ClickableObjectID id,float radius);
    void changeRadiusOfScrollableElement(ScrollableElementID id,float radius);
    void deleteClickableObject(ClickableObjectID);
    void deleteScrollableElement(ScrollableElementID);

    void activate();
    static bool isActive(){return singletonPtr_s != nullptr && getInstance().active;}
    void deactivate();
};
}
#endif // MOUSEAPI_H
